;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Teaser                                                        ;
;                                                               ;
; Copyright (c)2012 James Vernon                                ;
; Written and programmed by James Vernon                        ;
; jamesv82@live.com.au                                          ;
; http://www.jvti.org                                           ;
;                                                               ;
; You are allowed to use parts of this code as long as you give ;
; credit where it is due :)                                     ;
;                                                               ;
;===============================================================;

#include "ab5t.h"                               ; header file

;------------------------------------------------
; Start of program
;------------------------------------------------
start:
        set     textwrite,(iy+sgrflags)         ; write text to gbuf

        ld      hl,storyText                    ; HL => start of text data
        ld      b,(hl)                          ; B = # of pages of text to display
        inc     hl                              ; HL => first text page data

displayStory:
        push    bc                              ; save page counter
        push    hl                              ; save text data ptr
        bcall(_grbufclr)
        pop     hl                              ; restore text data ptr
        ld      de,0
        ld      (pencol),de                     ; put cursor to top left
        call    displayPage                     ; display page of text
        ld      b,128
        call    wait
        pop     bc                              ; restore page counter
        djnz    displayStory                    ; loop

displayLogo:
        bcall(_grbufclr)
        ld      hl,logoData                     ; HL => logo data
        ld      b,(hl)                          ; B = # lines to display
        inc     hl                              ; HL => actual picture data
        ld      de,gbuf+(23*12)                 ; HL => line on gbuf to start from
        ld      c,40                            ; C = # lines to perform cascade effect on

displayLogoLine:
        push    bc                              ; save counters
        ld      bc,12                           ; BC = 12 bytes per gbuf row
        ldir                                    ; copy picture line from (HL) to gbuf line (DE)
        pop     bc                              ; restore counters
        push    bc                              ; save counters
        push    hl                              ; save picture data ptr
        push    de                              ; save gbuf ptr
        ld      b,c                             ; B = # lines to perform cascade effect on
        ld      hl,-12
        add     hl,de                           ; HL => line on gbuf before (DE)

doCascadeEffect:
        push    bc                              ; save line counter
        ld      bc,12                           ; BC = 12 bytes per gbuf row
        ldir                                    ; copy picture line from gbuf line (HL) to gbuf line (DE)
        pop     bc                              ; restore line counter
        djnz    doCascadeEffect                 ; loop

        call    ionFastCopy                     ; display updated gbuf
        ei
        ld      b,8
        call    wait
        pop     de                              ; restore gbuf ptr
        pop     hl                              ; restory pic data ptr
        pop     bc                              ; restore counters
        dec     c                               ; dec effect counter
        djnz    displayLogoLine                 ; loop

        ld      hl,35*256+24
        ld      (pencol),hl
        ld      hl,strTheyreBack
        call    displayPage                     ; display "THEY'RE BACK..."

        call    ionFastCopy
        ld      b,8
        call    wait

        call    invertScreen
        ld      b,8
        call    wait

        call    invertScreen
        ld      b,12
        call    wait

        call    invertScreen
        ld      b,8
        call    wait

        call    invertScreen

        ld      b,64
        call    wait

scrollLogoUp:
        ld      b,20                            ; B = 20 times to scroll up a pixel

scrollLogoUpLoop:
        push    bc                              ; save counter
        ld      hl,gbuf+12                      ; HL => 2nd line of gbuf
        ld      de,gbuf                         ; DE => start of gbuf
        ld      bc,63*12                        ; 63 lines of 12 bytes to copy
        ldir
        ld      b,2
        call    wait
        call    ionFastCopy
        pop     bc
        djnz    scrollLogoUpLoop

displayCredits:
        ld      hl,strCredits                   ; HL => credit data
        ld      b,(hl)                          ; B = # lines
        inc     hl                              ; HL => first credit data

displayCreditsLoop:
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = penrow, pencol
        ld      (pencol),de                     ; set it
        inc     hl                              ; HL => actual string
        push    bc                              ; save counter
        bcall(_vputs)                           ; display string
        pop     bc                              ; restore counter
        djnz    displayCreditsLoop

        call    ionFastCopy
        call    waitKey

quit:
        res     textwrite,(iy+sgrflags)         ; write text directly to lcd
        ret

;------------------------------------------------
; waitKey - wait for a key press
;
; Input:    none
; Output:   A = key pressed (_getcsc style)
;------------------------------------------------
waitKey:
        ei                                      ; enable interrupts
        res     apdrunning,(iy+apdflags)        ; disable apd
wKeyLoop:
        halt
        bcall(_getcsc)                          ; A = key pressed
        or      a                               ; check if 0
        jr      z,wKeyLoop                      ; if so, no key was pressed
        set     apdrunning,(iy+apdflags)        ; enable apd
        ret

;------------------------------------------------
; displayChar - display a character to gbuf and update pencol & penrow accordingly
;
; Input:    HL => character to display, NULL means line feed
; Output:   HL => next character
;------------------------------------------------
displayChar:
        ld      a,(hl)                          ; A = character
        inc     hl                              ; HL => next character
        push    hl                              ; save character pointer
        or      a                               ; null?
        jr      z,displayCharLineFeed           ; if so, do line feed
        bcall(_vputmap)                         ; display character and update pencol & penrow accordingly
        pop     hl                              ; restore character pointer
        ret

displayCharLineFeed:
        ld      hl,(pencol)                     ; H = (penrow), L = (pencol)
        ld      l,0                             ; cursor to left of screen
        ld      a,7
        add     a,h
        ld      h,a                             ; cursor to next line
        ld      (pencol),hl                     ; save new pencol & penrow
        pop     hl                              ; restore character pointer (from earlier in displayChar)
        ld      b,10
        jr      wait                            ; wait for a little, then ret

;------------------------------------------------
; displayPage - display a page of text using the displayChar routine and update it to the lcd
;
; Input:    HL => string data, each line NULL terminated, page is terminated by $FF
; Output:   HL => next page string data
;------------------------------------------------
displayPage:
        ld      a,(hl)                          ; A = first character of line
        inc     a                               ; was it $FF
        jr      z,displayPageEndPage            ; if so, update ptr to next page data and ret
        call    displayChar                     ; display char to gbuf
        push    hl                              ; save char ptr
        call    ionFastCopy                     ; update lcd
        pop     hl                              ; restore char ptr
        ld      b,3
        call    wait                            ; wait for a short time before displaying next char
        jr      displayPage

displayPageEndPage:
        inc     hl                              ; HL => next page string data
        ret

;------------------------------------------------
; invertScreen - invert gbuf and display it
;
; Input:    none
; Output:   none
;------------------------------------------------
invertScreen:
        ld      bc,64*12                        ; BC = 64 rows of 12 bytes to invert
        ld      hl,gbuf                         ; HL => start of gbuf
invertScreenLoop:
        ld      a,(hl)
        cpl                                     ; invert byte
        ld      (hl),a                          ; and put it back to gbuf
        inc     hl                              ; inc gbuf ptr
        dec     bc                              ; dec counter
        ld      a,b
        or      c                               ; does BC = 0?
        jr      nz,invertScreenLoop
        jp      ionFastCopy                     ; show gbuf and ret

;------------------------------------------------
; wait - wait for a specified time
;
; Input:    B = number of times to double halt
; Output:   none
;------------------------------------------------
wait:
        ei
waitLoop:
        halt
        halt
        djnz    waitLoop
        ret

;------------------------------------------------
; Start of data
;------------------------------------------------
storyText:
.db     7                                       ; # of pages of text

.db     "Connecting to network...",0
.db     0,0
.db     "Welcome to Intex Systems",0
.db     0
.db     "Receiving transmission",0
.db     "data...",0
.db     $FF

.db     "During the peak of human",0
.db     "civilisation, as we reached",0
.db     "out to the stars looking for",0
.db     "answers to our own origin,",0
.db     "we came across something",0
.db     "we least expected.",0
.db     $FF

.db     "It started with one space",0
.db     "station, infested with an",0
.db     "alien race. Soon after,",0
.db     "human colonies on distant",0
.db     "planets were attacked.",0
.db     "Numerous battles ensued.",0
.db     $FF

.db     "As the conflict raged on,",0
.db     "the human race decided to",0
.db     "launch a bold series of",0
.db     "strikes against the enemy,",0
.db     "in what we hoped would be",0
.db     "the final assault.",0
.db     $FF

.db     "The war dragged on longer",0
.db     "than mankind anticipated,",0
.db     "but the attempt at a final",0
.db     "assault was somewhat",0
.db     "successful; the aliens",0
.db     "retreated back into the",0
.db     "depths of space.",0
.db     $FF

.db     "A decade has passed, and the",0
.db     "human race has slowly",0
.db     "begun to recover from the",0
.db     "long war. Now, a distress",0
.db     "signal has been received",0
.db     "from an outlying space",0
.db     "station. Mankind fears the",0
.db     "worst. . .",0
.db     $FF

.db     "Private Hudson, it is time",0
.db     "to do your duty and find",0
.db     "out if our worst fears have",0
.db     "been realised - that the",0
.db     "ALIEN BREED has returned!",0
.db     $FF

logoData:
.db     10                                  ; # lines to display
.db     $00,$03,$d8,$6f,$fc,$1f,$3e,$7f,$ff,$83,$f0,$00
.db     $00,$07,$d8,$6f,$fe,$1f,$bf,$7f,$ff,$c3,$f0,$00
.db     $00,$0c,$d8,$60,$07,$00,$00,$00,$00,$e0,$00,$00
.db     $00,$0f,$d8,$6f,$33,$1f,$3e,$79,$e6,$63,$e0,$00
.db     $00,$0f,$d8,$6f,$33,$1f,$bf,$79,$e6,$63,$f0,$00
.db     $00,$0c,$d8,$6c,$33,$19,$b3,$61,$86,$60,$30,$00
.db     $00,$0c,$dc,$6c,$33,$19,$b3,$61,$86,$60,$30,$00
.db     $00,$0c,$df,$6f,$b3,$1f,$b3,$7d,$f7,$e3,$f0,$00
.db     $00,$0c,$cf,$67,$b3,$1f,$33,$3c,$f7,$e3,$e0,$00
.db     $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

strTheyreBack:
.db     "THEY'RE BACK...",0
.db     $FF

strCredits:
.db     5
.dw     25*256+14
.db     "Based on the original",0
.dw     31*256+13
.db     "ALIEN BREED series by",0
.dw     37*256+35
.db     "TEAM 17",0
.dw     47*256+21
.db     "by James Vernon",0
.dw     54*256+26
.db     "WWW.JVTI.ORG",0

.end
